# frozen_string_literal: true

require 'logger'

module Zashoku
  module AnimeIO
    class AnimeIndexer
      EMPTY_PARAMS = { watched: false, watched_on: 0 }.freeze
      AVS_FILE = '.avs.yml'

      attr_accessor :avs, :force, :indexed

      def initialize
        @made_changes = false
        @avs = {}
        @force = false
        @indexed = {}
      end

      def made_changes?
        @made_changes
      end

      def made_changes!
        @made_changes = true
      end

      def update_avs(file)
        obj = indexed[file]
        dir = File.dirname(File.dirname(obj.path))
        @avs = load_avs(File.join(dir, AVS_FILE))
        @avs[file] ||= {}
        @avs[file][:watched]    = obj.watched?
        @avs[file][:watched_on] = obj.watched_on
        write_avs(File.join(dir, AVS_FILE))
      end

      def load_avs(file)
        File.exist?(file) ? YAML.load(File.open(file, 'r', &:read)) : {}
      end

      def write_avs(file)
        return unless Zashoku.conf.get(%w[avs_files create?])
        File.open(file, 'w') { |f| f.puts YAML.dump(@avs) }
      end

      def glob_files(dir)
        Dir.glob("#{dir}*/*.{#{Zashoku::AnimeIO::EXTENSIONS.join(',')}}")
      end

      def index_files(files)
        files.map do |file|
          Zashoku.logger.debug "indexing #{file}"

          @avs[file] = EMPTY_PARAMS.dup unless avs.key?(file)

          if indexed&.key?(file)
            @avs[file][:watched]    = indexed[file].watched?
            @avs[file][:watched_on] = indexed[file].watched_on
          end

          {
            file =>
              # we have already indexed this file
              # get its seen data
              if indexed&.key?(file) && File::Stat.new(file).mtime <= indexed[file].last_modified
                indexed[file]
              else
                made_changes!
                AnimeFile.new(file, @avs[file])
              end
          }
        end.compact.reduce(:merge) || {}
      end

      def index_dirs(dirs)
        dirs.map do |dir|
          next unless File.exist? dir
          @avs = load_avs(File.join(dir, AVS_FILE))

          i = index_files(glob_files(dir))

          write_avs(File.join(dir, AVS_FILE))

          i
        end.reduce(:merge)
      end

      def index(dirs)
        @made_changes = false
        indexed_ = index_dirs(dirs)

        made_changes! if indexed_.length != indexed.length

        @indexed = indexed_
      end
    end
  end
end
