# frozen_string_literal: true

require 'mkmf'
require 'fileutils'
require 'observer'
require 'listen'
require 'benchmark'

require_relative 'animefile'
require_relative 'animeseries'
require_relative 'anime_indexer'
# require_relative '../../util/util'
# require_relative '../../util/folder_listen'

module Zashoku
  module AnimeIO
    class AnimeIO < Zashoku::Controller
      include Zashoku::Util
      include Observable

      PLAYLIST = '/tmp/zashoku_list.m3u'

      def initialize
        @empty_hash        = make_empty_hash
        @dir_last_modified = {}
        @made_changes      = true
        @indexer           = AnimeIndexer.new
        load
        # p ref_changed!(verbose: false)
        listen
        index
        # Zashoku.conf.add_observer(self, :pref_changed!)
      end

      def index
        @indexer.index(dirs)
        changed!('items') if @indexer.made_changes?
      end

      def indexed
        @indexer.indexed
      end

      def changed!(e)
        save
        changed
        notify_observers(sender: 'animeio', event: e)
      end

      def pref_changed!
        @indexer.index(dirs)
        changed! if @indexer.changed?
        relisten
      end

      def listen
        @listener = Listen.to(*dirs, only: /\.mkv$/) do |mod, add, del|
          a = @indexer.index_files(add + mod)
          indexed.merge!(a).reject! { |p| del.include? p }
          changed!('items')
        end
        @listener.start
      end

      def animefile_changed!
        return unless @mpvbridge.what_changed[:local_anime]
        changed!
      end

      def dirs
        Zashoku.conf.get(['anime_locations']).split(':').collect do |dir|
          parsed_dir = Zashoku.conf.parse_dir(dir)
          parsed_dir if File.exist?(parsed_dir)
        end.compact
      end

      def make_empty_hash
        af = AnimeFile.new('empty')
        { AnimeSeries.new('empty', [af]) => [af] }
      end

      # returns the parent directory of a file
      def parent(file)
        File.basename(File.dirname(file))
      end

      # convert an AnimeFile array to a hash structured like (string)parent =>
      # (AnimeFile array)[child0, child1, child2] and return it
      def to_h(arr)
        keys = {}
        hash = {}

        arr.sort_by(&:title).each do |file|
          if keys.key?(parent(file.path))
            key = keys[parent(file.path)]
          else
            key = AnimeSeries.new(parent(file.path))
            keys[parent(file.path)] = key
            hash[key] = []
          end

          hash[key] << file
          key << file
        end
        hash.sort_by { |j| j[0].title }.to_h
      end

      # return the difference of getAired and getWatched
      def unwatched
        to_h(indexed.reject { |_f, a| a.watched? }.values)
      end

      # return an AnimeFile array of all mkvs in any dir specified in
      # $pref["dirs"]
      def items
        to_h(indexed.values)
      end

      def log_watched(file)
        file = Zashoku.conf.parse_dir(Zashoku.WATCH_LOG_TAG('watch_log'))
        open(file, 'a') do |f|
          f.puts "#{`date`.chomp} ./#{File.basename(file)} #{WATCH_LOG_TAG}"
        end
      end

      def watched!(file)
        return if indexed[file].watched?
        indexed[file].watched!
        changed!('items')
        @indexer.update_avs(file)
      end

      def recently_watched
        indexed.map { |k, v| { v.watched_on => k } if v.watched? }.compact
      end

      def unwatch(path = nil)
        path =
          if indexed[path]&.watched?
            path
          elsif !path
            r = recently_watched
            r.empty? ? nil : r.sort_by { |h| h.keys.first }.reverse[0].values[0]
          else
            nil
          end

        return unless path
        indexed[path].unwatch
        changed!('items')
        @indexer.update_avs(path)
      end

      def watch(file)
        Zashoku.command(mod: 'mpv', meth: :play, args: file)
      end

      def watch_list(files)
        File.open(PLAYLIST, 'w') { |f| files.each { |i| f.puts i.path } }

        Zashoku.command(mod: 'mpv', meth: :play, args: PLAYLIST)
      end

      def save
        Zashoku.conf.set(['local_anime'], Util.encode_object(indexed))
      end

      def load
        raw = Zashoku.conf.get(['local_anime'])
        @indexer.indexed =
          raw.class == String ? Util.decode_object(raw) : {}
      end
    end
  end
end
